#pragma NOIV               // Do not generate interrupt vectors
#pragma WARNINGLEVEL (0)

#include "M:\DEV\USB\COMMON FIRMWARE\fx2.h"
#include "M:\DEV\USB\COMMON FIRMWARE\fx2regs.h"
#include "M:\DEV\USB\COMMON FIRMWARE\fx2sdly.h"            // SYNCDELAY macro
#include <absacc.h>
#include <string.h>
#include "ACCESFX2USBVR_BAE.h"
#include "CARD_SPECIFIC.h"

// GEN = GenRegister()
// DEF = power-on-default
// WDG = Watchdog Timer
// MAP = digital Matrix Mapping
// PWM = Pulse Width Modified digital output
// DEB = Debounce DIO
// LATCH = high and low side pulse latching DIO
// CFW = Cypress Framework (common to all "C" ACCES USB devices)

//BYTE code bDEFLength = 0;
//BYTE xdata * pDEFLength = &bDEFLength;

//*************************************************************************************************
// these are CFW variables
//*************************************************************************************************
BYTE configuration;
BYTE alternatesetting;
BYTE code 	    bee_str[4];
BYTE xdata *    pee_str = bee_str;
BYTE I2C_Addr;
BYTE DB_Addr;
void EEPROMWriteByteInner(WORD addr, BYTE value); //prototype

//*************************************************************************************************
// these are WDG variables
//*************************************************************************************************
extern data WORD Seconds;                           // from FW.C
WORD Tenths;                           // from FW.C
bit fBark;
bit fActivity;

void TickActivity()
{
  static int blinkcounter = 0;
  if (blinkcounter) {if (--blinkcounter < _ACTIVITY_LED_ON_TIME_) SetLED();}
  else if (fActivity)
  {
    blinkcounter = _ACTIVITY_LED_TIME_;
    ClrLED();
    fActivity = FALSE;
    petWatchdog();
  }

	if (fBark)
		fActivity = TRUE;

}

void GEN_Register_VR(void)
{
    WORD l;

    if ( USB_READ ) // read DEFBuf, or the length of it.
		{
       l = LengthLSB | (LengthMSB << 8);
       if (l == 0) { EP0BCL = 0; return; }
       if ((IndexLSB | IndexMSB) == 0) ;  // any read with index==0 is a read of the DEF struct or its size
       else
       {
           if (IndexMSB == 0)
               switch (IndexLSB)
               {
                   case 1:         // read build revision
                       strncpy(EP0BUF+1, "V" __DATE__ "@" __TIME__ "\0", 30);
                       EP0BUF[0] = strlen(EP0BUF)+1;
                       EP0BCL = LengthLSB;
                       break;
                   case 2:
                       break;
               }
       }
    }
}

//*************************************************************************************************
// these are CFW functions
//*************************************************************************************************
extern code STRINGDSCR StringDscr2;
extern code STRINGDSCR StringDscr0;


void UpdateSN()  // copies 8 bytes from EP0 to SN field in DSCR.a51
{
	BYTE i = 0;
    EEPROMRead((WORD)0x1DF8,(WORD)8,(WORD)EP0BUF);

	for ( i=0; i<8; i++ )
	    *(BYTE xdata*)(&StringDscr0 + 1 + i) = EP0BUF[i];
}

void TD_Init(void)
{
    I2C_Addr = SERIAL_ADDR | ((I2CS & 0x10) >> 4);
    DB_Addr = (BOOL)(I2C_Addr & 0x01);

    CKCON |= bmBIT5;

    CS_PreInit();

    EZUSB_InitI2C();

    UpdateSN();
    CS_NameMomentInit();

    Rwuen = TRUE;

    SetLED();
    CS_PostInit();
}

void TD_Poll(void)
{
    CS_PrePoll();
  	TickActivity();
    doVbus();
    CS_PostPoll();
}

void OverwriteStringDscrM(char *newstr)
{
	BYTE i = 0;
	char len = strlen(newstr);

    ((BYTE xdata*)&StringDscr2)[0] = (len)*2+2;
	for ( i=0; i<len; i++ )
	    *(BYTE xdata*)(&StringDscr2 + 1 + i) = newstr[i];
}

void EEPROMWriteByteInner(WORD addr, BYTE value)
{
	BYTE i = 0;

	if(DB_Addr)
		pee_str[i++] = MSB(addr);

	pee_str[i++] = LSB(addr);
	pee_str[i++] = value;

	EZUSB_WriteI2C(I2C_Addr, i, pee_str);
    EZUSB_WaitForEEPROMWrite(I2C_Addr);
}

void EEPROMWriteByte(WORD addr, BYTE value)
{
    EEPROMWriteByteInner( addr, value);
}

void EEPROMWrite(WORD addr, BYTE length, BYTE xdata *buf)
{
	BYTE	i;
	for(i=0; i < length; ++i)
		EEPROMWriteByte(addr++, buf[i]);
}

void EEPROMRead(WORD addr, BYTE length, BYTE xdata *buf)
{
	BYTE		i = 0;
	BYTE		j = 0;

	if(DB_Addr)
		pee_str[i++] = MSB(addr);

	pee_str[i++] = LSB(addr);

	EZUSB_WriteI2C(I2C_Addr, i, pee_str);

	EZUSB_ReadI2C(I2C_Addr, length, buf);
}


void VRRAMEEPROM()
{
    WORD addr, len2, bc, i;
    addr = ValueLSB;		         // Get address and length
    addr |= ValueMSB << 8;
    len2 = LengthLSB;
    len2 |= LengthMSB << 8;
    // Is this an upload command ?
    if(SETUPDAT[0] == VR_UPLOAD)
    {
    	while(len2)					// Move requested data through EP0IN
    	{							// one packet at a time.
            while(EP0CS & bmEPBUSY);
    		if(len2 < EP0BUFF_SIZE) bc = len2; else bc = EP0BUFF_SIZE;
    		if(SETUPDAT[1] == VR_RAM)    		// Is this a RAM upload ?
    		{
    		    if ((IndexLSB == 0x01) && (IndexMSB == 0x00) && (ValueMSB & 0x80))       // supports GenRegister check
    			{
    			    EP0BUF[0] = 0x03;
    				EP0BUF[1] = 1; // indicate "REV E PCB"
                    EP0BUF[2] = 3;
    			}

    			else for(i=0; i<bc; i++) *(EP0BUF+i) = *((BYTE xdata *)addr+i);
    			;
    		}
    		else
    		{
    			for(i=0; i<bc; i++) *(EP0BUF+i) = 0xcd;
    			EEPROMRead(addr, (BYTE)bc, EP0BUF);
    		}

    		EP0BCH = 0;
    		EP0BCL = (BYTE)bc; // Arm endpoint with # bytes to transfer

    		addr += bc;
    		len2 -= bc;
    	}
    }
    // Is this a download command ?
    else if(SETUPDAT[0] == VR_DOWNLOAD)
    {
    	while(len2)					// Move new data through EP0OUT
    	{							// one packet at a time.
    		// Arm endpoint - do it here to clear (after sud avail)
    		EP0BCH = 0;
    		EP0BCL = 0; // Clear bytecount to allow new data in; also stops NAKing

    		while(EP0CS & bmEPBUSY);

    		bc = EP0BCL; // Get the new bytecount

    		// Is this a RAM download ?
    		if(SETUPDAT[1] == VR_RAM)
    		{
    			for(i=0; i<bc; i++)
    				 *((BYTE xdata *)addr+i) = *(EP0BUF+i);
    		}
    		else
    			EEPROMWrite(addr, bc, EP0BUF);

    		addr += bc;
    		len2 -= bc;
    	}
    }
}

BOOL TD_Suspend(void) 			// Called before the device goes into suspend mode
{
	return (TRUE);
}

BOOL TD_Resume(void) 			// Called after the device resumes
{
	return(TRUE);
}

BOOL DR_GetDescriptor(void)
{
   return(TRUE);
}

BOOL DR_SetConfiguration(void)   // Called when a Set Configuration command is received
{
   Configuration = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetConfiguration(void)   // Called when a Get Configuration command is received
{
   EP0BUF[0] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_SetInterface(void)       // Called when a Set Interface command is received
{
   AlternateSetting = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetInterface(void)       // Called when a Set Interface command is received
{
   EP0BUF[0] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_GetStatus(void)
{
   return(TRUE);
}

BOOL DR_ClearFeature(void)
{
   return(TRUE);
}

BOOL DR_SetFeature(void)
{
   return(TRUE);
}

void PNP_QuerySimple_VR()
{
    BYTE k=0;
    if (USB_READ)
    {
        k++; // reserve length field
        EP0BUF[k++] = DEFREV_MINOR;
        EP0BUF[k++] = DEFREV_MAJOR;
        EP0BUF[k++] = ADC_SetConfigByteCount;
        EP0BUF[k++] = DIO_ControlByteCount;
        EP0BUF[k++] = DIO_ByteCount;
        EP0BUF[k++] = CTR_NumBlocks;
        EP0BUF[k++] = DAC_WordCount;
        EP0BUF[k++] = DEB_Supported; //indicate ALL bits DEBounce
        EP0BUF[k++] = LATCH_Supported; //indicate ALL bits LATCH
        EP0BUF[k++] = PWM_MAX_TABLELENGTH;
        EP0BUF[k++] = MAP_MAX_TABLELENGTH;
        EP0BUF[k++] = _CARD_SPECIFIC_MAP_HIGHESTSRCBITINDEX_;
        EP0BUF[k++] = _CARD_SPECIFIC_MAP_HIGHESTDESTBITINDEX_;
        EP0BUF[0] = k; // stuff length field
        EP0BCH = 0;
        EP0BCL = LengthLSB;
    }
}

BOOL HandleFW20VRs()
{
  BYTE bCounterStatus;
	BYTE i, index, b, ci;

	switch(SETUPDAT[1])
	{
      case AUR_GEN_REGISTER:
             GEN_Register_VR();
             break;
      case VR_RAM:
      case VR_EEPROM:
            VRRAMEEPROM();
	  		break;
      case AUR_QuerySimplePNP:
            PNP_QuerySimple_VR();
            break;

#if (DIO_ByteCount)


      default:
                return 0x00;      // VR not handled by this function
              break;
	}
	return 0x01;
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//	The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
   SUDPTRCTL |= bmSDPAUTO;
   GotSUD = TRUE;            // Set flag
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUDAV;         // Clear SUDAV IRQ
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUTOK;         // Clear SUTOK IRQ
}

void ISR_Sof(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSOF;            // Clear SOF IRQ
}

void ISR_Ures(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
      pConfigDscr = pFullSpeedConfigDscr;
      pOtherConfigDscr = pHighSpeedConfigDscr;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ
}

void ISR_Susp(void) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUSP;
}

void ISR_Highspeed(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
      pConfigDscr = pFullSpeedConfigDscr;
      pOtherConfigDscr = pHighSpeedConfigDscr;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmHSGRANT;
}

void ISR_Ep0ack(void) interrupt 0
{
}
void ISR_Stub(void) interrupt 0
{
}
void ISR_Ep0in(void) interrupt 0
{
}
void ISR_Ep0out(void) interrupt 0
{
}
void ISR_Ep1in(void) interrupt 0
{
}
void ISR_Ep1out(void) interrupt 0
{
}
void ISR_Ep2inout(void) interrupt 0
{
}
void ISR_Ep4inout(void) interrupt 0
{
}
void ISR_Ep6inout(void) interrupt 0
{
}
void ISR_Ep8inout(void) interrupt 0
{
}
void ISR_Ibn(void) interrupt 0
{
}
void ISR_Ep0pingnak(void) interrupt 0
{
}
void ISR_Ep1pingnak(void) interrupt 0
{
}
void ISR_Ep2pingnak(void) interrupt 0
{
}
void ISR_Ep4pingnak(void) interrupt 0
{
}
void ISR_Ep6pingnak(void) interrupt 0
{
}
void ISR_Ep8pingnak(void) interrupt 0
{
}
void ISR_Errorlimit(void) interrupt 0
{
}
void ISR_Ep2piderror(void) interrupt 0
{
}
void ISR_Ep4piderror(void) interrupt 0
{
}
void ISR_Ep6piderror(void) interrupt 0
{
}
void ISR_Ep8piderror(void) interrupt 0
{
}
void ISR_Ep2pflag(void) interrupt 0
{
}
void ISR_Ep4pflag(void) interrupt 0
{
}
void ISR_Ep6pflag(void) interrupt 0
{
}
void ISR_Ep8pflag(void) interrupt 0
{
}
void ISR_Ep2eflag(void) interrupt 0
{
}
void ISR_Ep4eflag(void) interrupt 0
{
}
void ISR_Ep6eflag(void) interrupt 0
{
}
void ISR_Ep8eflag(void) interrupt 0
{
}
void ISR_Ep2fflag(void) interrupt 0
{
}
void ISR_Ep4fflag(void) interrupt 0
{
}
void ISR_Ep6fflag(void) interrupt 0
{
}
void ISR_Ep8fflag(void) interrupt 0
{
}
void ISR_GpifComplete(void) interrupt 0
{
#ifdef HOOK_GPIFCOMPLETE
    Hook_GPIF_Complete();
#endif
}
void ISR_GpifWaveform(void) interrupt 0
{
}

